@@ -6,7 +6,8 @@ import tempfile
 from zipfile import BadZipfile
 from bs4 import BeautifulSoup
 
-from ..dwca import DwCAReader, GBIFResultsReader, DwCALine
+from ..dwca import DwCAReader, GBIFResultsReader
+from ..lines import DwCACoreLine
 from ..darwincore.utils import qualname as qn
 
 
@@ -14,11 +15,12 @@ from ..darwincore.utils import qualname as qn
 def _sample_data_path(filename):
     return os.path.join(os.path.dirname(__file__), 'sample_files', filename)
 
+# TODO: Move other class constants here so they can be used in any test
+GBIF_RESULTS_PATH = _sample_data_path('gbif-results.zip')
+
 
 class TestGBIFResultsReader(unittest.TestCase):
     """Unit tests for GBIFResultsReader class."""
-
-    GBIF_RESULTS_PATH = _sample_data_path('gbif-results.zip')
     MISSINGMETA_PATH = _sample_data_path('gbif-results-lacks-s-metadata.zip')
 
     CITATIONS_CONTENT = """Please cite this data as follows, and pay attention
@@ -28,8 +30,8 @@ class TestGBIFResultsReader(unittest.TestCase):
 Rights as supplied: Not supplied"""
 
     def test_dwcareader_features(self):
-        """Ensure we didn't break basic DwCAReader features."""
-        with GBIFResultsReader(self.GBIF_RESULTS_PATH) as results_dwca:
+        """Ensure we didn't break inherited basic DwCAReader features."""
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as results_dwca:
             self.assertEqual(158, len(results_dwca.lines))
             self.assertEqual('http://rs.tdwg.org/dwc/terms/Occurrence',
                              results_dwca.core_rowtype)
@@ -41,16 +43,16 @@ Rights as supplied: Not supplied"""
     # Specific GBIFResultsReader feature
     def test_citations_access(self):
         """Check the content of citations.txt is accessible."""
-        with GBIFResultsReader(self.GBIF_RESULTS_PATH) as results_dwca:
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as results_dwca:
             self.assertEqual(self.CITATIONS_CONTENT, results_dwca.citations)
 
     def test_rights_access(self):
         """Check the content of rights.txt is accessible."""
-        with GBIFResultsReader(self.GBIF_RESULTS_PATH) as results_dwca:
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as results_dwca:
             self.assertEqual(self.RIGHTS_CONTENT, results_dwca.rights)
 
     def test_source_metadata(self):
-        with GBIFResultsReader(self.GBIF_RESULTS_PATH) as results:
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as results:
             # We have 23 EML files in dataset/
             self.assertEqual(23, len(results.source_metadata))
             # Assert a key is present
@@ -71,8 +73,8 @@ Rights as supplied: Not supplied"""
                              'Rob')
 
     def test_line_source_metadata(self):
-        with GBIFResultsReader(self.GBIF_RESULTS_PATH) as results:
-            first_line = results.get_line('607759330')
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as results:
+            first_line = results.get_line_by_id('607759330')
             m = first_line.source_metadata
 
             self.assertIsInstance(m, BeautifulSoup)
@@ -80,7 +82,7 @@ Rights as supplied: Not supplied"""
                              individualName.givenName.contents[0],
                              'Stanley')
 
-            last_line = results.get_line('782700656')
+            last_line = results.get_line_by_id('782700656')
             m = last_line.source_metadata
 
             self.assertIsInstance(m, BeautifulSoup)
@@ -91,11 +93,12 @@ Rights as supplied: Not supplied"""
         with GBIFResultsReader(self.MISSINGMETA_PATH) as results:
             # We have source metadata, but not for all datasets/line...
             # We sould have None in this cases
-            first_line = results.get_line('607759330')
+            first_line = results.get_line_by_id('607759330')
             self.assertEqual(None, first_line.source_metadata)
 
 
 class TestDwCAReader(unittest.TestCase):
+    # TODO: Move line-oriented tests to another test class
     """Unit tests for DwCAReader class."""
 
     BASIC_ARCHIVE_PATH = _sample_data_path('dwca-simple-test-archive.zip')
@@ -104,8 +107,46 @@ class TestDwCAReader(unittest.TestCase):
     DEFAULT_VAL_PATH = _sample_data_path('dwca-test-default.zip')
     EXTENSION_ARCHIVE_PATH = _sample_data_path('dwca-star-test-archive.zip')
     MULTIEXTENSIONS_ARCHIVE_PATH = _sample_data_path('dwca-2extensions.zip')
+    IDS_ARCHIVE_PATH = _sample_data_path('dwca-ids.zip')
     UTF8EOL_ARCHIVE_PATH = _sample_data_path('dwca-utf8-eol-test.zip')
 
+    def test_line_human_representation(self):
+        with DwCAReader(self.BASIC_ARCHIVE_PATH) as basic_dwca:
+            l = basic_dwca.lines[0]
+            l_repr = str(l)
+            self.assertIn("Rowtype: http://rs.tdwg.org/dwc/terms/Occurrence", l_repr)
+            self.assertIn("Source: Core file", l_repr)
+            self.assertIn("Line id:", l_repr)
+            self.assertIn("Reference extension lines: No", l_repr)
+            self.assertIn("Reference source metadata: No", l_repr)
+            self.assertIn("http://rs.tdwg.org/dwc/terms/scientificName': u'tetraodon fluviatilis'",
+                          l_repr)
+
+        with DwCAReader(self.EXTENSION_ARCHIVE_PATH) as star_dwca:
+            l = star_dwca.lines[0]
+            l_repr = str(l)
+            self.assertIn("Rowtype: http://rs.tdwg.org/dwc/terms/Taxon", l_repr)
+            self.assertIn("Source: Core file", l_repr)
+            self.assertIn("Line id: 1", l_repr)
+            self.assertIn("Reference extension lines: Yes", l_repr)
+            self.assertIn("Reference source metadata: No", l_repr)
+
+            extension_l_repr = str(l.extensions[0])
+            self.assertIn("Rowtype: http://rs.gbif.org/terms/1.0/VernacularName", extension_l_repr)
+            self.assertIn("Source: Extension file", extension_l_repr)
+            self.assertIn("Core Line id: 1", extension_l_repr)
+            self.assertIn("ostrich", extension_l_repr)
+            self.assertIn("Reference extension lines: No", extension_l_repr)
+            self.assertIn("Reference source metadata: No", extension_l_repr)
+
+        with GBIFResultsReader(GBIF_RESULTS_PATH) as gbif_dwca:
+            l = gbif_dwca.lines[0]
+            l_repr = str(l)
+
+            self.assertIn("Rowtype: http://rs.tdwg.org/dwc/terms/Occurrence", l_repr)
+            self.assertIn("Source: Core file", l_repr)
+            self.assertIn("Reference source metadata: Yes", l_repr)
+
     def test_absolute_temporary_path(self):
         """Test the absolute_temporary_path() method."""
         with DwCAReader(self.BASIC_ARCHIVE_PATH) as dwca:
@@ -226,10 +267,21 @@ class TestDwCAReader(unittest.TestCase):
             self.assertEqual(2, len([l for l in dwca.each_line()]))
 
     def test_iterate_dwcalines(self):
-        """Test the each_line() method allows iterating over DwCALines"""
+        """Test the each_line() method allows iterating over DwCACoreLines"""
         with DwCAReader(self.BASIC_ARCHIVE_PATH) as dwca:
             for line in dwca.each_line():
-                self.assertIsInstance(line, DwCALine)
+                self.assertIsInstance(line, DwCACoreLine)
+
+    def test_iterate_order(self):
+        """Test that the order of the core file is respected when iterating with each_line()."""
+        # This is also probably tested inderectly elsewhere, but this is the right place :)
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
+            l = list(dwca.each_line())
+            # Lines are ordered like this in core: id 4-1-3-2
+            self.assertEqual(int(l[0].id), 4)
+            self.assertEqual(int(l[1].id), 1)
+            self.assertEqual(int(l[2].id), 3)
+            self.assertEqual(int(l[3].id), 2)
 
     def test_iterate_multiple_calls(self):
         with DwCAReader(self.MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
@@ -237,38 +289,52 @@ class TestDwCAReader(unittest.TestCase):
             # The second time, we can still find 4 lines...
             self.assertEqual(4, len([l for l in dwca.each_line()]))
 
+    def test_get_line_by_index(self):
+        """Test the get_line_by_index() method work as expected"""
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
+            # Lines are ordered like this in core: id 4-1-3-2
+            first_line = dwca.get_line_by_index(0)
+            self.assertEqual(4, int(first_line.id))
+
+            last_line = dwca.get_line_by_index(3)
+            self.assertEqual(2, int(last_line.id))
+
+            # None returned if bigger than archive (last index: 3)
+            self.assertIsNone(dwca.get_line_by_index(4))
+            self.assertIsNone(dwca.get_line_by_index(1000))
+
     def test_get_line_by_id_string(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
-        with DwCAReader(self.MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
             # Number can be passed as a string....
-            l = dwca.get_line('3')
+            l = dwca.get_line_by_id('3')
             self.assertEqual('Peliperdix', l.data[genus_qn])
 
     def test_get_line_by_id_multiple_calls(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
-        with DwCAReader(self.MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
-            l = dwca.get_line('3')
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
+            l = dwca.get_line_by_id('3')
             self.assertEqual('Peliperdix', l.data[genus_qn])
 
             # If iterator is not properly reset, None will be returned
             # the second time
-            l = dwca.get_line('3')
+            l = dwca.get_line_by_id('3')
             self.assertEqual('Peliperdix', l.data[genus_qn])
 
     def test_get_line_by_id_other(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
-        with DwCAReader(self.MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
-            # .Passed as an integer, conversion will be tried...
-            l = dwca.get_line(3)
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
+            # Passed as an integer, conversion will be tried...
+            l = dwca.get_line_by_id(3)
             self.assertEqual('Peliperdix', l.data[genus_qn])
 
     def test_get_inexistent_line(self):
-        """ Ensure get_line() returns None if we ask it an unexistent line. """
-        with DwCAReader(self.MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
-            self.assertEqual(None, dwca.get_line(8000))
+        """ Ensure get_line_by_id() returns None if we ask it an unexistent line. """
+        with DwCAReader(self.IDS_ARCHIVE_PATH) as dwca:
+            self.assertEqual(None, dwca.get_line_by_id(8000))
 
     def test_read_core_value(self):
         """Retrieve a simple value from core file"""
